@@ -18,7 +18,7 @@ from functools import reduce
 from operator import and_, or_
 from subprocess import PIPE, Popen, call
 from time import time
-from typing import IO, Any, Dict, Iterable, Iterator, List, Optional, Sequence, Set, TypeVar  # noqa F401
+from typing import IO, Any, Dict, Iterable, Iterator, List, Optional, Sequence, Set, Tuple, TypeVar  # noqa F401
 
 import apt
 import six
@@ -28,6 +28,7 @@ from univention.config_registry import ConfigRegistry
 from univention.testing.codes import TestCodes
 from univention.testing.errors import TestError
 from univention.testing.internal import UCSVersion
+from univention.testing.pytest import PytestRunner
 
 __all__ = ['TestEnvironment', 'TestCase', 'TestResult', 'TestFormatInterface']
 
@@ -217,6 +218,9 @@ class Check(object):
 		"""Check if precondition to run test is met."""
 		raise NotImplementedError()
 
+	def pytest_args(self, environment):  # type: (TestEnvironment) -> List[str]
+		return []
+
 
 class CheckExecutable(Check):
 
@@ -231,7 +235,7 @@ class CheckExecutable(Check):
 	def check(self, _environment):  # type: (TestEnvironment) -> Iterator[Verdict]
 		"""Check environment for required executable."""
 		if not os.path.isabs(self.filename):
-			if self.filename.startswith('python'):
+			if self.filename.startswith('python') or self.filename.startswith('pytest') or self.filename.startswith('py.test'):
 				self.filename = '/usr/bin/' + self.filename
 			elif self.filename.endswith('sh'):
 				self.filename = '/bin/' + self.filename
@@ -302,6 +306,20 @@ class CheckTags(Check):
 			else:
 				yield Verdict(Verdict.ERROR, 'De-selected by tag: %s' % (' '.join(environment.tags_required),), TestCodes.REASON_ROLE_MISMATCH)
 
+	def pytest_args(self, environment):  # type: (TestEnvironment) -> List[str]
+		args = []
+		for tag in self.tags:
+			args.extend(['--ucs-test-default-tags', tag])
+		for tag in (environment.tags_required or []):
+			if tag in ('SKIP', 'WIP'):
+				continue
+			args.extend(['--ucs-test-tags-required', tag])
+		for tag in (environment.tags_prohibited or []):
+			if tag in ('SKIP', 'WIP'):
+				continue
+			args.extend(['--ucs-test-tags-prohibited', tag])
+		return args
+
 
 class CheckRoles(Check):
 
@@ -478,6 +496,18 @@ class CheckExposure(Check):
 		else:
 			yield Verdict(Verdict.INFO, 'Safe enough: %s <= %s' % (exposure, environment.exposure))
 
+	def pytest_args(self, environment):  # type: (TestEnvironment) -> List[str]
+		args = []
+		args.extend(['--ucs-test-exposure', environment.exposure.lower()])
+		exposure = 'DANGEROUS'
+		try:
+			exposure, expected_md5 = self.exposure.split(None, 1)
+		except ValueError:
+			exposure = self.exposure
+		if exposure:
+			args.extend(['--ucs-test-default-exposure', exposure.lower()])
+		return args
+
 
 class TestCase(object):
 
@@ -497,6 +527,7 @@ class TestCase(object):
 		self.otrs = set()  # type: Set[str]
 		self.timeout = None  # type: Optional[int]
 		self.signaled = None  # type: Optional[int]
+		self.is_pytest = False  # type: bool
 
 	def load(self):  # type: () -> TestCase
 		"""
@@ -563,6 +594,7 @@ class TestCase(object):
 				exc_info=True)
 			raise TestError(ex)
 
+		self.is_pytest = PytestRunner.is_pytest(self)
 		return self
 
 	def check(self, environment):  # type: (TestEnvironment) -> List[Check]
@@ -582,15 +614,28 @@ class TestCase(object):
 		conditions += list(self.exposure.check(environment))
 		return conditions
 
+	def pytest_check(self, environment):  # type: (TestEnvironment) -> List[str]
+		args = []
+		args += self.exe.pytest_args(environment)
+		args += self.versions.pytest_args(environment)
+		args += self.tags.pytest_args(environment)
+		args += self.roles.pytest_args(environment)
+		args += self.components.pytest_args(environment)
+		args += self.packages.pytest_args(environment)
+		args += self.exposure.pytest_args(environment)
+		return args
+
 	def _run_tee(self, proc, result, stdout=sys.stdout, stderr=sys.stderr):
+		# type: (Popen, TestResult, IO[str], IO[str]) -> None
 		"""Run test collecting and passing through stdout, stderr:"""
-		EOF = b''
+		assert proc.stdout
+		assert proc.stderr
 		channels = {
 			proc.stdout.fileno(): (proc.stdout, [], u'stdout', stdout, b'[]', bytearray()),
 			proc.stderr.fileno(): (proc.stderr, [], u'stderr', stderr, b'()', bytearray()),
-		}
+		}  # type: Dict[int, Tuple[IO[str], List, str, IO[str], bytes, bytearray]]
 		combined = []
-		next_kill = next_read = None
+		next_kill = next_read = 0.0
 		shutdown = False
 		kill_sequence = self._terminate_proc(proc)
 		while channels:
@@ -598,44 +643,43 @@ class TestCase(object):
 			if self.signaled == signal.SIGALRM:
 				if next_kill <= current:
 					try:
-						next_kill = current + kill_sequence.next()
+						next_kill = current + next(kill_sequence)
 					except StopIteration:
 						shutdown = True
 						next_kill = current + 1.0
 			elif self.signaled == signal.SIGCHLD:
 				shutdown = True
 				next_kill = current + 1.0
-			try:
-				delay = max(0.0, min(filter(None, (next_kill, next_read))) - current)
-			except ValueError:
-				delay = None
 
+			delays = [max(0.0, t - current) for t in (next_kill, next_read) if t > 0.0]
 			try:
-				rlist, _wlist, _elist = select.select(channels.keys(), [], [], delay)
+				rlist, _wlist, _elist = select.select(list(channels), [], [], min(delays) if delays else None)
 			except select.error as ex:
 				if ex.args[0] == errno.EINTR:
 					TestCase.logger.debug('select() interrupted by SIG%d rc=%r', self.signaled, proc.poll())
 					continue
 				raise
 
-			next_read = None
-			for fd in rlist or channels.keys():
+			next_read = 0.0
+			for fd in rlist or list(channels):
 				stream, log, name, out, paren, buf = channels[fd]
 
 				if fd in rlist:
 					data = os.read(fd, 1024)
 					if six.PY3:
-						out.buffer.write(data)
+						out.buffer.write(data)  # type: ignore
 					else:
 						out.write(data)
 					buf += data
+					eof = data == b''
 				else:
-					data = EOF if shutdown else None
+					data = b''
+					eof = shutdown
 
 				while buf:
-					if data == EOF:
+					if eof:
 						line = buf
-						buf = None
+						buf = bytearray()
 					else:
 						match = TestCase.RE_NL.search(buf)
 						if not match:
@@ -648,7 +692,7 @@ class TestCase(object):
 					log.append(entry)
 					combined.append(entry)
 
-				if data == EOF:
+				if eof:
 					stream.close()
 					del channels[fd]
 					TestCase._attach(result, name, log)
@@ -713,6 +757,10 @@ class TestCase(object):
 		dirname = os.path.dirname(self.filename)
 		cmd = [self.exe.filename, base] + self.args
 
+		if self.is_pytest:
+			cmd = PytestRunner.extend_command(self, cmd)
+			cmd.extend(self.pytest_check(result.environment))
+
 		time_start = datetime.now()
 
 		print('\n*** BEGIN *** %r ***' % (
@@ -812,6 +860,7 @@ class TestResult(object):
 		self.artifacts = {}
 		self.condition = None
 		self.eofs = None
+		self.is_pytest = False
 
 	def dump(self, stream=sys.stdout):
 		"""Dump test result data."""
