@@ -1,6 +1,8 @@
 # vim: set fileencoding=utf-8 ft=python sw=4 ts=4 :
 """Format UCS Test results as JUnit report."""
 
+from __future__ import print_function
+
 import errno
 import os
 import sys
@@ -13,6 +15,7 @@ import six
 
 from univention.testing.codes import TestCodes
 from univention.testing.data import TestCase, TestFormatInterface, TestResult  # noqa F401
+from univention.testing.format.text import Raw
 
 __all__ = ['Junit']
 
@@ -34,11 +37,19 @@ class Junit(TestFormatInterface):
 		super(Junit, self).__init__(stream)
 		self.outdir = "test-reports"
 		self.now = datetime.today()
+		self.raw = Raw(stream)
 
 	def begin_test(self, case, prefix=''):  # type: (TestCase, str) -> None
 		"""Called before each test."""
 		super(Junit, self).begin_test(case, prefix)
 		self.now = datetime.today().replace(microsecond=0)
+		print('\r', end='', file=self.stream)
+		self.raw.begin_test(case, prefix)
+		self.stream.flush()
+
+	def end_run(self):
+		print('')  # clear \r
+		self.stream.flush()
 
 	def end_test(self, result):  # type: (TestResult) -> None
 		"""Called after each test."""
@@ -58,6 +69,9 @@ class Junit(TestFormatInterface):
 			classname = classname[:-3]
 
 		filename = os.path.join(self.outdir, '%s.xml' % (result.case.uid,))
+		if result.case.is_pytest and os.path.exists(filename):
+			return  # pytest itself already writes the junit file! create one if pytest did not
+
 		dirname = os.path.dirname(filename)
 		try:
 			os.makedirs(dirname)
